from contextlib import contextmanager
from typing import Any

from ape.api import ConverterAPI, Web3Provider
from ape.exceptions import NetworkError, ProviderError
from ape.types import AddressType
from web3.main import ENS


class ENSConversions(ConverterAPI):
    """Converts ENS names like `my-name.eth` to `0xAbCd...1234`"""

    def is_convertible(self, value: Any) -> bool:
        if not isinstance(value, str):
            return False

        elif "." not in value:
            return False

        elif not ENS.is_valid_name(value):
            return False

        else:
            try:
                with self._connect_to_ens() as ens:
                    return ens.address(value) is not None
            except (NetworkError, ProviderError):
                return False

    def convert(self, value: str) -> AddressType:
        with self._connect_to_ens() as ens:
            return ens.address(value)

    @contextmanager
    def _connect_to_ens(self):
        def _get_ens_from_provider(provider_):
            if not isinstance(provider_, Web3Provider):
                raise NotImplementedError("Currently, only web3 providers work with this plugin.")

            web3 = provider_._web3

            if not hasattr(web3, "ens"):
                raise NotImplementedError("This provider does not implement ENS calls.")

            return web3.ens

        provider = self.network_manager.active_provider
        if provider and provider.network.name == "mainnet":
            yield _get_ens_from_provider(provider)

        else:
            with self.network_manager.parse_network_choice("ethereum:mainnet") as provider:
                yield _get_ens_from_provider(provider)



from functools import partial
from pathlib import Path

import pytest
from ape.api import Web3Provider

from ape_ens.converter import ENSConversions


@pytest.fixture
def provider_class(mocker):
    class MockMainnetProvider(Web3Provider):
        name = "mock"
        provider_settings = {}
        data_folder = Path(".")
        request_header = {}

        def connect(self):
            self._web3 = mocker.MagicMock()

        def disconnect(self):
            self._web3 = None  # type: ignore

    return MockMainnetProvider


@pytest.fixture
def converter(provider_class):
    ens = ENSConversions()
    mainnet = ens.network_manager.ethereum.mainnet
    provider = partial(provider_class, network=mainnet)
    mainnet.providers["mock"] = provider
    mainnet.set_default_provider("mock")
    return ens


def test_is_convertible(converter):
    assert converter.is_convertible("test.eth")


def test_is_not_convertible(converter):
    assert not converter.is_convertible("test")
    assert not converter.is_convertible("0xe1122aa5533228143C4Ce8fC4642aa33b857B332")
    assert not converter.is_convertible(23452345)
